# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo
from .route_list_item_param import RouteListItemParam
from .split_config_semantic_page_item_class_param import SplitConfigSemanticPageItemClassParam

__all__ = [
    "ActionTypeConfigUpdateParams",
    "UpsertTransformConfig",
    "UpsertRouteConfig",
    "UpsertSplitConfig",
    "UpsertSplitConfigPrintPageSplitConfig",
    "UpsertSplitConfigSemanticPageSplitConfig",
    "UpsertJoinConfig",
    "UpsertEmailConfig",
]


class UpsertTransformConfig(TypedDict, total=False):
    complex_tabular_transform_enabled: Annotated[bool, PropertyInfo(alias="complexTabularTransformEnabled")]
    """Whether complex tabular transforms are enabled on the pipeline.

    This enables the pipeline to parse CSVs with multiple tables in the same file,
    and to transpose CSVs that can't be parsed row-wise.
    """

    independent_document_processing_enabled: Annotated[bool, PropertyInfo(alias="independentDocumentProcessingEnabled")]
    """Whether independent transformations is enabled.

    For PDFs sent through the pipeline, this enables independent transformations for
    each individual page. For CSVs, this enables transforming chunks of rows in the
    CSV.
    """

    name: str

    next_action_type_config_id: Annotated[str, PropertyInfo(alias="nextActionTypeConfigID")]
    """Unique identifier of action type config to run after transformation.

    Currently only email is supported.
    """

    output_schema: Annotated[object, PropertyInfo(alias="outputSchema")]
    """Desired output structure defined in standard JSON Schema convention."""

    output_schema_name: Annotated[str, PropertyInfo(alias="outputSchemaName")]
    """Name of output schema object."""


class UpsertRouteConfig(TypedDict, total=False):
    description: str
    """Description of router.

    Can be used to provide additional context on router's purpose and expected
    inputs.
    """

    name: str

    routes: Iterable[RouteListItemParam]
    """List of routes."""


class UpsertSplitConfig(TypedDict, total=False):
    name: str

    print_page_split_config: Annotated[
        UpsertSplitConfigPrintPageSplitConfig, PropertyInfo(alias="printPageSplitConfig")
    ]

    semantic_page_split_config: Annotated[
        UpsertSplitConfigSemanticPageSplitConfig, PropertyInfo(alias="semanticPageSplitConfig")
    ]

    split_type: Annotated[str, PropertyInfo(alias="splitType")]


class UpsertSplitConfigPrintPageSplitConfig(TypedDict, total=False):
    next_action_type_config_id: Annotated[str, PropertyInfo(alias="nextActionTypeConfigID")]
    """The unique ID of the action type configuration you want to use for this action."""


class UpsertSplitConfigSemanticPageSplitConfig(TypedDict, total=False):
    item_classes: Annotated[Iterable[SplitConfigSemanticPageItemClassParam], PropertyInfo(alias="itemClasses")]


class UpsertJoinConfig(TypedDict, total=False):
    join_type: Annotated[Literal["standard"], PropertyInfo(alias="joinType")]
    """The type of join to perform."""

    name: str

    next_action_type_config_id: Annotated[str, PropertyInfo(alias="nextActionTypeConfigID")]
    """Unique identifier of action type config to run after join."""

    output_schema: Annotated[object, PropertyInfo(alias="outputSchema")]
    """Desired output structure defined in standard JSON Schema convention."""

    output_schema_name: Annotated[str, PropertyInfo(alias="outputSchemaName")]
    """Name of output schema object."""


class UpsertEmailConfig(TypedDict, total=False):
    body: str
    """Body of the email.

    This can be HTML, and include template variables in the form of
    `{{template_variable}}`. Template variables are taken from the output of the
    transformation.
    """

    from_email: Annotated[str, PropertyInfo(alias="fromEmail")]
    """Email address to send the email from."""

    from_name: Annotated[str, PropertyInfo(alias="fromName")]
    """Name of the sender."""

    name: str

    subject: str
    """Subject of the email.

    This can include template variables in the form of `{{template_variable}}`.
    Template variables are taken from the output of the transformation.
    """

    to_email: Annotated[str, PropertyInfo(alias="toEmail")]
    """Email address to send the email to."""

    to_name: Annotated[str, PropertyInfo(alias="toName")]
    """Name of the recipient."""


ActionTypeConfigUpdateParams: TypeAlias = Union[
    UpsertTransformConfig, UpsertRouteConfig, UpsertSplitConfig, UpsertJoinConfig, UpsertEmailConfig
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["EventListParams"]


class EventListParams(TypedDict, total=False):
    action_type_config_ids: Annotated[List[str], PropertyInfo(alias="actionTypeConfigIDs")]

    ending_before: Annotated[str, PropertyInfo(alias="endingBefore")]
    """A cursor to use in pagination.

    `endingBefore` is an event ID that defines your place in the list.
    """

    event_ids: Annotated[List[str], PropertyInfo(alias="eventIDs")]

    event_types: Annotated[
        List[Literal["transform", "route", "split_collection", "split_item", "error", "join"]],
        PropertyInfo(alias="eventTypes"),
    ]

    limit: int

    reference_ids: Annotated[List[str], PropertyInfo(alias="referenceIDs")]

    sort_order: Annotated[Literal["asc", "desc"], PropertyInfo(alias="sortOrder")]
    """Specifies sorting behavior.

    The two options are `asc` and `desc` to sort ascending and descending
    respectively, with default sort being ascending. Paging works in both
    directions.
    """

    starting_after: Annotated[str, PropertyInfo(alias="startingAfter")]
    """A cursor to use in pagination.

    `startingAfter` is an event ID that defines your place in the list.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import subscription_list_params, subscription_create_params, subscription_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.subscription import Subscription
from ..types.subscription_list_response import SubscriptionListResponse

__all__ = ["SubscriptionsResource", "AsyncSubscriptionsResource"]


class SubscriptionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SubscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return SubscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return SubscriptionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        action_type_config_id: str,
        name: str,
        type: Literal["transform", "route", "split_collection", "split_item", "error", "join"],
        disabled: bool | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """
        Creates a new subscription to listen to transform or error events.

        Args:
          action_type_config_id: Unique identifier of action this subscription listens to.

          name: Name of subscription.

          type: Type of subscription.

          disabled: Toggles whether subscription is active or not.

          webhook_url: URL bem will send webhook requests to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1-alpha/subscriptions",
            body=maybe_transform(
                {
                    "action_type_config_id": action_type_config_id,
                    "name": name,
                    "type": type,
                    "disabled": disabled,
                    "webhook_url": webhook_url,
                },
                subscription_create_params.SubscriptionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    def retrieve(
        self,
        subscription_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """
        Get a Subscription

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return self._get(
            f"/v1-alpha/subscriptions/{subscription_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    def update(
        self,
        subscription_id: str,
        *,
        action_type_config_id: str | NotGiven = NOT_GIVEN,
        disabled: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        type: Literal["transform", "route", "split_collection", "split_item", "error", "join"] | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """Updates an existing subscription.

        Follow conventional PATCH behavior, so only
        included fields will be updated.

        Args:
          action_type_config_id: Unique identifier of action this subscription listens to.

          disabled: Toggles whether subscription is active or not.

          name: Name of subscription.

          type: Type of subscription.

          webhook_url: URL bem will send webhook requests to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return self._patch(
            f"/v1-alpha/subscriptions/{subscription_id}",
            body=maybe_transform(
                {
                    "action_type_config_id": action_type_config_id,
                    "disabled": disabled,
                    "name": name,
                    "type": type,
                    "webhook_url": webhook_url,
                },
                subscription_update_params.SubscriptionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    def list(
        self,
        *,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubscriptionListResponse:
        """List Subscriptions

        Args:
          ending_before: A cursor to use in pagination.

        `endingBefore` is a task ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, starting with `sub_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call
              can include `endingBefore=sub_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the previous
              page of the list.

          limit: This specifies a limit on the number of objects to return, ranging between 1
              and 100.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is a task ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, ending with `sub_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can
              include `startingAfter=sub_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the next page
              of the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1-alpha/subscriptions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    subscription_list_params.SubscriptionListParams,
                ),
            ),
            cast_to=SubscriptionListResponse,
        )

    def delete(
        self,
        subscription_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Deletes an existing subscription.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1-alpha/subscriptions/{subscription_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncSubscriptionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSubscriptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubscriptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubscriptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return AsyncSubscriptionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        action_type_config_id: str,
        name: str,
        type: Literal["transform", "route", "split_collection", "split_item", "error", "join"],
        disabled: bool | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """
        Creates a new subscription to listen to transform or error events.

        Args:
          action_type_config_id: Unique identifier of action this subscription listens to.

          name: Name of subscription.

          type: Type of subscription.

          disabled: Toggles whether subscription is active or not.

          webhook_url: URL bem will send webhook requests to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1-alpha/subscriptions",
            body=await async_maybe_transform(
                {
                    "action_type_config_id": action_type_config_id,
                    "name": name,
                    "type": type,
                    "disabled": disabled,
                    "webhook_url": webhook_url,
                },
                subscription_create_params.SubscriptionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    async def retrieve(
        self,
        subscription_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """
        Get a Subscription

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return await self._get(
            f"/v1-alpha/subscriptions/{subscription_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    async def update(
        self,
        subscription_id: str,
        *,
        action_type_config_id: str | NotGiven = NOT_GIVEN,
        disabled: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        type: Literal["transform", "route", "split_collection", "split_item", "error", "join"] | NotGiven = NOT_GIVEN,
        webhook_url: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Subscription:
        """Updates an existing subscription.

        Follow conventional PATCH behavior, so only
        included fields will be updated.

        Args:
          action_type_config_id: Unique identifier of action this subscription listens to.

          disabled: Toggles whether subscription is active or not.

          name: Name of subscription.

          type: Type of subscription.

          webhook_url: URL bem will send webhook requests to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        return await self._patch(
            f"/v1-alpha/subscriptions/{subscription_id}",
            body=await async_maybe_transform(
                {
                    "action_type_config_id": action_type_config_id,
                    "disabled": disabled,
                    "name": name,
                    "type": type,
                    "webhook_url": webhook_url,
                },
                subscription_update_params.SubscriptionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Subscription,
        )

    async def list(
        self,
        *,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SubscriptionListResponse:
        """List Subscriptions

        Args:
          ending_before: A cursor to use in pagination.

        `endingBefore` is a task ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, starting with `sub_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call
              can include `endingBefore=sub_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the previous
              page of the list.

          limit: This specifies a limit on the number of objects to return, ranging between 1
              and 100.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is a task ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, ending with `sub_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can
              include `startingAfter=sub_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the next page
              of the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1-alpha/subscriptions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    subscription_list_params.SubscriptionListParams,
                ),
            ),
            cast_to=SubscriptionListResponse,
        )

    async def delete(
        self,
        subscription_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Deletes an existing subscription.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not subscription_id:
            raise ValueError(f"Expected a non-empty value for `subscription_id` but received {subscription_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1-alpha/subscriptions/{subscription_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class SubscriptionsResourceWithRawResponse:
    def __init__(self, subscriptions: SubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = to_raw_response_wrapper(
            subscriptions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            subscriptions.retrieve,
        )
        self.update = to_raw_response_wrapper(
            subscriptions.update,
        )
        self.list = to_raw_response_wrapper(
            subscriptions.list,
        )
        self.delete = to_raw_response_wrapper(
            subscriptions.delete,
        )


class AsyncSubscriptionsResourceWithRawResponse:
    def __init__(self, subscriptions: AsyncSubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = async_to_raw_response_wrapper(
            subscriptions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            subscriptions.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            subscriptions.update,
        )
        self.list = async_to_raw_response_wrapper(
            subscriptions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            subscriptions.delete,
        )


class SubscriptionsResourceWithStreamingResponse:
    def __init__(self, subscriptions: SubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = to_streamed_response_wrapper(
            subscriptions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            subscriptions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            subscriptions.update,
        )
        self.list = to_streamed_response_wrapper(
            subscriptions.list,
        )
        self.delete = to_streamed_response_wrapper(
            subscriptions.delete,
        )


class AsyncSubscriptionsResourceWithStreamingResponse:
    def __init__(self, subscriptions: AsyncSubscriptionsResource) -> None:
        self._subscriptions = subscriptions

        self.create = async_to_streamed_response_wrapper(
            subscriptions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            subscriptions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            subscriptions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            subscriptions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            subscriptions.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import TaskStatus, task_list_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.task import Task
from .._base_client import make_request_options
from ..types.task_status import TaskStatus
from ..types.task_list_response import TaskListResponse

__all__ = ["TasksResource", "AsyncTasksResource"]


class TasksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return TasksResourceWithStreamingResponse(self)

    def retrieve(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Task:
        """
        Get a Task

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._get(
            f"/v1-alpha/tasks/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Task,
        )

    def list(
        self,
        *,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        status: TaskStatus | NotGiven = NOT_GIVEN,
        task_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TaskListResponse:
        """List Tasks

        Args:
          ending_before: A cursor to use in pagination.

        `endingBefore` is a task ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, starting with `tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call
              can include `endingBefore=tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the previous
              page of the list.

          limit: This specifies a limit on the number of objects to return, ranging between 1
              and 100.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is a task ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, ending with `tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can
              include `startingAfter=tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the next page
              of the list.

          status: Array with the task statuses, formatted as a CSV array.

          task_ids: Array with the task IDs, formatted as a CSV array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1-alpha/tasks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                        "status": status,
                        "task_ids": task_ids,
                    },
                    task_list_params.TaskListParams,
                ),
            ),
            cast_to=TaskListResponse,
        )


class AsyncTasksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return AsyncTasksResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Task:
        """
        Get a Task

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._get(
            f"/v1-alpha/tasks/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Task,
        )

    async def list(
        self,
        *,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        status: TaskStatus | NotGiven = NOT_GIVEN,
        task_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TaskListResponse:
        """List Tasks

        Args:
          ending_before: A cursor to use in pagination.

        `endingBefore` is a task ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, starting with `tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call
              can include `endingBefore=tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the previous
              page of the list.

          limit: This specifies a limit on the number of objects to return, ranging between 1
              and 100.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is a task ID that defines your
              place in the list. For example, if you make a list request and receive 50
              objects, ending with `tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can
              include `startingAfter=tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the next page
              of the list.

          status: Array with the task statuses, formatted as a CSV array.

          task_ids: Array with the task IDs, formatted as a CSV array.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1-alpha/tasks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                        "status": status,
                        "task_ids": task_ids,
                    },
                    task_list_params.TaskListParams,
                ),
            ),
            cast_to=TaskListResponse,
        )


class TasksResourceWithRawResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.retrieve = to_raw_response_wrapper(
            tasks.retrieve,
        )
        self.list = to_raw_response_wrapper(
            tasks.list,
        )


class AsyncTasksResourceWithRawResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.retrieve = async_to_raw_response_wrapper(
            tasks.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            tasks.list,
        )


class TasksResourceWithStreamingResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.retrieve = to_streamed_response_wrapper(
            tasks.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            tasks.list,
        )


class AsyncTasksResourceWithStreamingResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.retrieve = async_to_streamed_response_wrapper(
            tasks.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            tasks.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PipelineListParams"]


class PipelineListParams(TypedDict, total=False):
    ending_before: Annotated[str, PropertyInfo(alias="endingBefore")]
    """A cursor to use in pagination.

    `endingBefore` is a pipeline ID that defines your place in the list. For
    example, if you make a list request and receive 50 objects, starting with
    `pl_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can include
    `endingBefore=pl_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the previous page of the
    list.
    """

    limit: int

    starting_after: Annotated[str, PropertyInfo(alias="startingAfter")]
    """A cursor to use in pagination.

    `startingAfter` is a pipeline ID that defines your place in the list. For
    example, if you make a list request and receive 50 objects, ending with
    `pl_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can include
    `startingAfter=pl_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the next page of the
    list.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .action_type_config import ActionTypeConfig

__all__ = ["Pipeline"]


class Pipeline(BaseModel):
    complex_tabular_transform_enabled: bool = FieldInfo(alias="complexTabularTransformEnabled")
    """Whether complex tabular transforms are enabled on the pipeline.

    This enables the pipeline to parse CSVs with multiple tables in the same file,
    and to transpose CSVs that can't be parsed row-wise.
    """

    email_address: str = FieldInfo(alias="emailAddress")
    """Email address automatically created by bem.

    You can forward emails with or without attachments, to be transformed.
    """

    independent_document_processing_enabled: bool = FieldInfo(alias="independentDocumentProcessingEnabled")
    """Whether independent transformations is enabled.

    For PDFs sent through the pipeline, this enables independent transformations for
    each individual page. For CSVs, this enables transforming chunks of rows in the
    CSV.
    """

    name: str
    """Name of pipeline"""

    output_schema: object = FieldInfo(alias="outputSchema")
    """Desired output structure defined in standard JSON Schema convention."""

    output_schema_name: str = FieldInfo(alias="outputSchemaName")
    """Name of output schema object."""

    pipeline_id: str = FieldInfo(alias="pipelineID")
    """The unique identifier of the pipeline."""

    action_config: Optional[ActionTypeConfig] = FieldInfo(alias="actionConfig", default=None)
    """
    Configuration of router that maps names of routes to respective pipelines to
    route to.
    """

    webhook_enabled: Optional[bool] = FieldInfo(alias="webhookEnabled", default=None)
    """DEPRECATED - use subscriptions for webhook events.

    Whether webhook functionality is enabled.
    """

    webhook_url: Optional[str] = FieldInfo(alias="webhookURL", default=None)
    """DEPRECATED - use subscriptions for webhook events.

    URL bem will send webhook requests to if webhooks are enabled for the pipeline.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TransformationDeleteResponse"]


class TransformationDeleteResponse(BaseModel):
    delete_count: Optional[int] = FieldInfo(alias="deleteCount", default=None)
    """count of transformations successfully deleted."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .pipeline import Pipeline

__all__ = ["PipelineListResponse"]

PipelineListResponse: TypeAlias = List[Pipeline]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Subscription"]


class Subscription(BaseModel):
    name: str
    """Name of subscription."""

    subscription_id: str = FieldInfo(alias="subscriptionID")
    """The unique identifier of the subscription."""

    type: Literal["transform", "route", "split_collection", "split_item", "error", "join"]
    """Type of subscription."""

    action_type_config_id: Optional[str] = FieldInfo(alias="actionTypeConfigID", default=None)
    """Unique identifier of action subscription listens to.

    Only associated with pipeline IDs at the moment.
    """

    disabled: Optional[bool] = None
    """Toggles whether subscription is active or not."""

    webhook_url: Optional[str] = FieldInfo(alias="webhookURL", default=None)
    """URL bem will send webhook requests to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated

from .._utils import PropertyInfo
from .action_type_config_upsert_base_param import ActionTypeConfigUpsertBaseParam

__all__ = ["UpsertEmailConfigParam"]


class UpsertEmailConfigParam(ActionTypeConfigUpsertBaseParam, total=False):
    body: str
    """Body of the email.

    This can be HTML, and include template variables in the form of
    `{{template_variable}}`. Template variables are taken from the output of the
    transformation.
    """

    from_email: Annotated[str, PropertyInfo(alias="fromEmail")]
    """Email address to send the email from."""

    from_name: Annotated[str, PropertyInfo(alias="fromName")]
    """Name of the sender."""

    subject: str
    """Subject of the email.

    This can include template variables in the form of `{{template_variable}}`.
    Template variables are taken from the output of the transformation.
    """

    to_email: Annotated[str, PropertyInfo(alias="toEmail")]
    """Email address to send the email to."""

    to_name: Annotated[str, PropertyInfo(alias="toName")]
    """Name of the recipient."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, Annotated, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo
from .._models import set_pydantic_config
from .input_type import InputType

__all__ = ["RouteActionBaseParam"]


class RouteActionBaseParam(TypedDict, total=False):
    input_content: Required[Annotated[Union[str, Base64FileInput], PropertyInfo(alias="inputContent", format="base64")]]
    """The file content you want to route as a base64 encoded string.

    If the `inputType` is `email`, this is equivalent to the raw format returned by
    the Gmail API.
    """

    input_type: Required[Annotated[InputType, PropertyInfo(alias="inputType")]]
    """The input type of the content you're sending for transformation."""


set_pydantic_config(RouteActionBaseParam, {"arbitrary_types_allowed": True})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["RouteListItem", "Origin", "OriginEmail", "Regex"]


class OriginEmail(BaseModel):
    patterns: Optional[List[str]] = None
    """
    Regular expression to match against the email address, using the Go (RE2) regex
    syntax.
    """


class Origin(BaseModel):
    email: Optional[OriginEmail] = None


class Regex(BaseModel):
    patterns: Optional[List[str]] = None
    """Regular expression to match against the input, using the Go (RE2) regex syntax."""


class RouteListItem(BaseModel):
    name: str
    """Name of route.

    Should indicate what the input type or intent is to appropriately analyze input.
    """

    action_type_config_id: Optional[str] = FieldInfo(alias="actionTypeConfigID", default=None)
    """ID of action type config to run after routing.

    Currently only Transform Configs are supported.
    """

    description: Optional[str] = None
    """Description of route.

    Can be used to provide additional context on route's purpose.
    """

    origin: Optional[Origin] = None
    """The origin of the route."""

    regex: Optional[Regex] = None
    """Regex to match against the input."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .task_status import TaskStatus

__all__ = ["TaskListParams"]


class TaskListParams(TypedDict, total=False):
    ending_before: Annotated[str, PropertyInfo(alias="endingBefore")]
    """A cursor to use in pagination.

    `endingBefore` is a task ID that defines your place in the list. For example, if
    you make a list request and receive 50 objects, starting with
    `tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can include
    `endingBefore=tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the previous page of the
    list.
    """

    limit: int
    """
    This specifies a limit on the number of objects to return, ranging between 1
    and 100.
    """

    sort_order: Annotated[Literal["asc", "desc"], PropertyInfo(alias="sortOrder")]
    """Specifies sorting behavior.

    The two options are `asc` and `desc` to sort ascending and descending
    respectively, with default sort being ascending. Paging works in both
    directions.
    """

    starting_after: Annotated[str, PropertyInfo(alias="startingAfter")]
    """A cursor to use in pagination.

    `startingAfter` is a task ID that defines your place in the list. For example,
    if you make a list request and receive 50 objects, ending with
    `tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK`, your subsequent call can include
    `startingAfter=tsk_2c9AXIj48cUYJtCuv1gsQtHGDzK` to fetch the next page of the
    list.
    """

    status: TaskStatus
    """Array with the task statuses, formatted as a CSV array."""

    task_ids: Annotated[List[str], PropertyInfo(alias="taskIDs")]
    """Array with the task IDs, formatted as a CSV array."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from .upsert_join_config_param import UpsertJoinConfigParam
from .upsert_email_config_param import UpsertEmailConfigParam
from .upsert_route_config_param import UpsertRouteConfigParam
from .upsert_split_config_param import UpsertSplitConfigParam
from .upsert_transform_config_param import UpsertTransformConfigParam
from .action_type_config_create_base_param import ActionTypeConfigCreateBaseParam

__all__ = [
    "ActionTypeConfigCreateParams",
    "CreateTransformConfig",
    "CreateTransformConfigBody",
    "CreateRouteConfig",
    "CreateRouteConfigBody",
    "CreateSplitConfig",
    "CreateSplitConfigBody",
    "CreateJoinConfig",
    "CreateJoinConfigBody",
    "CreateEmailConfig",
    "CreateEmailConfigBody",
]


class CreateTransformConfig(TypedDict, total=False):
    body: Required[CreateTransformConfigBody]


class CreateTransformConfigBody(ActionTypeConfigCreateBaseParam, UpsertTransformConfigParam, total=False):
    pass


class CreateRouteConfig(TypedDict, total=False):
    body: Required[CreateRouteConfigBody]


class CreateRouteConfigBody(ActionTypeConfigCreateBaseParam, UpsertRouteConfigParam, total=False):
    pass


class CreateSplitConfig(TypedDict, total=False):
    body: Required[CreateSplitConfigBody]


class CreateSplitConfigBody(ActionTypeConfigCreateBaseParam, UpsertSplitConfigParam, total=False):
    pass


class CreateJoinConfig(TypedDict, total=False):
    body: Required[CreateJoinConfigBody]


class CreateJoinConfigBody(ActionTypeConfigCreateBaseParam, UpsertJoinConfigParam, total=False):
    pass


class CreateEmailConfig(TypedDict, total=False):
    body: Required[CreateEmailConfigBody]


class CreateEmailConfigBody(ActionTypeConfigCreateBaseParam, UpsertEmailConfigParam, total=False):
    pass


ActionTypeConfigCreateParams: TypeAlias = Union[
    CreateTransformConfig, CreateRouteConfig, CreateSplitConfig, CreateJoinConfig, CreateEmailConfig
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import TypeAlias

__all__ = ["AnyType"]

AnyType: TypeAlias = Union[List[object], str, float, bool, object, None]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated

from .._utils import PropertyInfo
from .action_type_config_upsert_base_param import ActionTypeConfigUpsertBaseParam

__all__ = ["UpsertTransformConfigParam"]


class UpsertTransformConfigParam(ActionTypeConfigUpsertBaseParam, total=False):
    complex_tabular_transform_enabled: Annotated[bool, PropertyInfo(alias="complexTabularTransformEnabled")]
    """Whether complex tabular transforms are enabled on the pipeline.

    This enables the pipeline to parse CSVs with multiple tables in the same file,
    and to transpose CSVs that can't be parsed row-wise.
    """

    independent_document_processing_enabled: Annotated[bool, PropertyInfo(alias="independentDocumentProcessingEnabled")]
    """Whether independent transformations is enabled.

    For PDFs sent through the pipeline, this enables independent transformations for
    each individual page. For CSVs, this enables transforming chunks of rows in the
    CSV.
    """

    next_action_type_config_id: Annotated[str, PropertyInfo(alias="nextActionTypeConfigID")]
    """Unique identifier of action type config to run after transformation.

    Currently only email is supported.
    """

    output_schema: Annotated[object, PropertyInfo(alias="outputSchema")]
    """Desired output structure defined in standard JSON Schema convention."""

    output_schema_name: Annotated[str, PropertyInfo(alias="outputSchemaName")]
    """Name of output schema object."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["InputType"]

InputType: TypeAlias = Literal[
    "email",
    "pdf",
    "xlsx",
    "xls",
    "docx",
    "csv",
    "text",
    "json",
    "png",
    "jpeg",
    "heic",
    "heif",
    "webp",
    "html",
    "wav",
    "mp3",
    "m4a",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from bem import BemSDK, AsyncBemSDK
from bem.types import (
    Subscription,
    SubscriptionListResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSubscriptions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: BemSDK) -> None:
        subscription = client.subscriptions.create(
            action_type_config_id="actionTypeConfigID",
            name="name",
            type="transform",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: BemSDK) -> None:
        subscription = client.subscriptions.create(
            action_type_config_id="actionTypeConfigID",
            name="name",
            type="transform",
            disabled=True,
            webhook_url="https://bem-example.ai/test/url",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: BemSDK) -> None:
        response = client.subscriptions.with_raw_response.create(
            action_type_config_id="actionTypeConfigID",
            name="name",
            type="transform",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: BemSDK) -> None:
        with client.subscriptions.with_streaming_response.create(
            action_type_config_id="actionTypeConfigID",
            name="name",
            type="transform",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: BemSDK) -> None:
        subscription = client.subscriptions.retrieve(
            "subscriptionID",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: BemSDK) -> None:
        response = client.subscriptions.with_raw_response.retrieve(
            "subscriptionID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: BemSDK) -> None:
        with client.subscriptions.with_streaming_response.retrieve(
            "subscriptionID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            client.subscriptions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: BemSDK) -> None:
        subscription = client.subscriptions.update(
            subscription_id="subscriptionID",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: BemSDK) -> None:
        subscription = client.subscriptions.update(
            subscription_id="subscriptionID",
            action_type_config_id="actionTypeConfigID",
            disabled=True,
            name="name",
            type="transform",
            webhook_url="https://bem-example.ai/test/url",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: BemSDK) -> None:
        response = client.subscriptions.with_raw_response.update(
            subscription_id="subscriptionID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: BemSDK) -> None:
        with client.subscriptions.with_streaming_response.update(
            subscription_id="subscriptionID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            client.subscriptions.with_raw_response.update(
                subscription_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: BemSDK) -> None:
        subscription = client.subscriptions.list()
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: BemSDK) -> None:
        subscription = client.subscriptions.list(
            ending_before="endingBefore",
            limit=1,
            sort_order="asc",
            starting_after="startingAfter",
        )
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: BemSDK) -> None:
        response = client.subscriptions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: BemSDK) -> None:
        with client.subscriptions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_delete(self, client: BemSDK) -> None:
        subscription = client.subscriptions.delete(
            "subscriptionID",
        )
        assert subscription is None

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_delete(self, client: BemSDK) -> None:
        response = client.subscriptions.with_raw_response.delete(
            "subscriptionID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert subscription is None

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_delete(self, client: BemSDK) -> None:
        with client.subscriptions.with_streaming_response.delete(
            "subscriptionID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert subscription is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_delete(self, client: BemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            client.subscriptions.with_raw_response.delete(
                "",
            )


class TestAsyncSubscriptions:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncBemSDK) -> None:
        subscription = await async_client.subscriptions.create(
            action_type_config_id="actionTypeConfigID",
            name="name",
            type="transform",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncBemSDK) -> None:
        subscription = await async_client.subscriptions.create(
            action_type_config_id="actionTypeConfigID",
            name="name",
            type="transform",
            disabled=True,
            webhook_url="https://bem-example.ai/test/url",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.subscriptions.with_raw_response.create(
            action_type_config_id="actionTypeConfigID",
            name="name",
            type="transform",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncBemSDK) -> None:
        async with async_client.subscriptions.with_streaming_response.create(
            action_type_config_id="actionTypeConfigID",
            name="name",
            type="transform",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncBemSDK) -> None:
        subscription = await async_client.subscriptions.retrieve(
            "subscriptionID",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.subscriptions.with_raw_response.retrieve(
            "subscriptionID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncBemSDK) -> None:
        async with async_client.subscriptions.with_streaming_response.retrieve(
            "subscriptionID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            await async_client.subscriptions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncBemSDK) -> None:
        subscription = await async_client.subscriptions.update(
            subscription_id="subscriptionID",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncBemSDK) -> None:
        subscription = await async_client.subscriptions.update(
            subscription_id="subscriptionID",
            action_type_config_id="actionTypeConfigID",
            disabled=True,
            name="name",
            type="transform",
            webhook_url="https://bem-example.ai/test/url",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.subscriptions.with_raw_response.update(
            subscription_id="subscriptionID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncBemSDK) -> None:
        async with async_client.subscriptions.with_streaming_response.update(
            subscription_id="subscriptionID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            await async_client.subscriptions.with_raw_response.update(
                subscription_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncBemSDK) -> None:
        subscription = await async_client.subscriptions.list()
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncBemSDK) -> None:
        subscription = await async_client.subscriptions.list(
            ending_before="endingBefore",
            limit=1,
            sort_order="asc",
            starting_after="startingAfter",
        )
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.subscriptions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncBemSDK) -> None:
        async with async_client.subscriptions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionListResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_delete(self, async_client: AsyncBemSDK) -> None:
        subscription = await async_client.subscriptions.delete(
            "subscriptionID",
        )
        assert subscription is None

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncBemSDK) -> None:
        response = await async_client.subscriptions.with_raw_response.delete(
            "subscriptionID",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert subscription is None

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncBemSDK) -> None:
        async with async_client.subscriptions.with_streaming_response.delete(
            "subscriptionID",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert subscription is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncBemSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            await async_client.subscriptions.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

from .route_list_item_param import RouteListItemParam
from .action_type_config_upsert_base_param import ActionTypeConfigUpsertBaseParam

__all__ = ["UpsertRouteConfigParam"]


class UpsertRouteConfigParam(ActionTypeConfigUpsertBaseParam, total=False):
    description: str
    """Description of router.

    Can be used to provide additional context on router's purpose and expected
    inputs.
    """

    routes: Iterable[RouteListItemParam]
    """List of routes."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo
from .action_type_config_param import ActionTypeConfigParam

__all__ = ["PipelineUpdateParams"]


class PipelineUpdateParams(TypedDict, total=False):
    action_config: Annotated[ActionTypeConfigParam, PropertyInfo(alias="actionConfig")]
    """
    Configuration of router that maps names of routes to respective pipelines to
    route to.
    """

    complex_tabular_transform_enabled: Annotated[bool, PropertyInfo(alias="complexTabularTransformEnabled")]
    """Whether complex tabular transforms are enabled on the pipeline.

    This enables the pipeline to parse CSVs with multiple tables in the same file,
    and to transpose CSVs that can't be parsed row-wise.
    """

    independent_document_processing_enabled: Annotated[bool, PropertyInfo(alias="independentDocumentProcessingEnabled")]
    """Whether independent transformations is enabled.

    For PDFs sent through the pipeline, this enables independent transformations for
    each individual page. For CSVs, this enables transforming chunks of rows in the
    CSV.
    """

    name: str
    """Name of pipeline"""

    output_schema: Annotated[object, PropertyInfo(alias="outputSchema")]
    """Desired output structure defined in standard JSON Schema convention.

    Note - We DO NOT support non-alphanumeric characters in names of fields.
    """

    output_schema_name: Annotated[str, PropertyInfo(alias="outputSchemaName")]
    """Name of output schema object."""

    webhook_enabled: Annotated[bool, PropertyInfo(alias="webhookEnabled")]
    """DEPRECATED - use subscriptions for webhook events.

    Whether webhook functionality is enabled.
    """

    webhook_url: Annotated[str, PropertyInfo(alias="webhookURL")]
    """DEPRECATED - use subscriptions for webhook events.

    URL bem will send webhook requests to with successful transformation outputs if
    webhooks are enabled for the pipeline.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .task import Task as Task
from .event import Event as Event
from .any_type import AnyType as AnyType
from .pipeline import Pipeline as Pipeline
from .event_base import EventBase as EventBase
from .input_type import InputType as InputType
from .action_base import ActionBase as ActionBase
from .action_type import ActionType as ActionType
from .task_status import TaskStatus as TaskStatus
from .subscription import Subscription as Subscription
from .webhook_secret import WebhookSecret as WebhookSecret
from .route_list_item import RouteListItem as RouteListItem
from .join_action_base import JoinActionBase as JoinActionBase
from .task_list_params import TaskListParams as TaskListParams
from .email_action_base import EmailActionBase as EmailActionBase
from .event_list_params import EventListParams as EventListParams
from .route_action_base import RouteActionBase as RouteActionBase
from .split_action_base import SplitActionBase as SplitActionBase
from .action_list_params import ActionListParams as ActionListParams
from .action_type_config import ActionTypeConfig as ActionTypeConfig
from .task_list_response import TaskListResponse as TaskListResponse
from .event_list_response import EventListResponse as EventListResponse
from .action_create_params import ActionCreateParams as ActionCreateParams
from .get_actions_response import GetActionsResponse as GetActionsResponse
from .pipeline_list_params import PipelineListParams as PipelineListParams
from .route_list_item_param import RouteListItemParam as RouteListItemParam
from .transform_action_base import TransformActionBase as TransformActionBase
from .join_action_base_param import JoinActionBaseParam as JoinActionBaseParam
from .pipeline_create_params import PipelineCreateParams as PipelineCreateParams
from .pipeline_list_response import PipelineListResponse as PipelineListResponse
from .pipeline_update_params import PipelineUpdateParams as PipelineUpdateParams
from .action_type_config_base import ActionTypeConfigBase as ActionTypeConfigBase
from .email_action_base_param import EmailActionBaseParam as EmailActionBaseParam
from .route_action_base_param import RouteActionBaseParam as RouteActionBaseParam
from .split_action_base_param import SplitActionBaseParam as SplitActionBaseParam
from .action_create_base_param import ActionCreateBaseParam as ActionCreateBaseParam
from .action_type_config_param import ActionTypeConfigParam as ActionTypeConfigParam
from .create_action_base_param import CreateActionBaseParam as CreateActionBaseParam
from .subscription_list_params import SubscriptionListParams as SubscriptionListParams
from .upsert_join_config_param import UpsertJoinConfigParam as UpsertJoinConfigParam
from .upsert_email_config_param import UpsertEmailConfigParam as UpsertEmailConfigParam
from .upsert_route_config_param import UpsertRouteConfigParam as UpsertRouteConfigParam
from .upsert_split_config_param import UpsertSplitConfigParam as UpsertSplitConfigParam
from .pipeline_retrieve_response import PipelineRetrieveResponse as PipelineRetrieveResponse
from .subscription_create_params import SubscriptionCreateParams as SubscriptionCreateParams
from .subscription_list_response import SubscriptionListResponse as SubscriptionListResponse
from .subscription_update_params import SubscriptionUpdateParams as SubscriptionUpdateParams
from .transformation_list_params import TransformationListParams as TransformationListParams
from .action_correct_route_params import ActionCorrectRouteParams as ActionCorrectRouteParams
from .transform_action_base_param import TransformActionBaseParam as TransformActionBaseParam
from .transformation_create_params import TransformationCreateParams as TransformationCreateParams
from .transformation_delete_params import TransformationDeleteParams as TransformationDeleteParams
from .transformation_list_response import TransformationListResponse as TransformationListResponse
from .transformation_update_params import TransformationUpdateParams as TransformationUpdateParams
from .action_type_config_base_param import ActionTypeConfigBaseParam as ActionTypeConfigBaseParam
from .upsert_transform_config_param import UpsertTransformConfigParam as UpsertTransformConfigParam
from .action_type_config_list_params import ActionTypeConfigListParams as ActionTypeConfigListParams
from .transformation_create_response import TransformationCreateResponse as TransformationCreateResponse
from .transformation_delete_response import TransformationDeleteResponse as TransformationDeleteResponse
from .update_transformation_response import UpdateTransformationResponse as UpdateTransformationResponse
from .action_type_config_create_params import ActionTypeConfigCreateParams as ActionTypeConfigCreateParams
from .action_type_config_list_response import ActionTypeConfigListResponse as ActionTypeConfigListResponse
from .action_type_config_update_params import ActionTypeConfigUpdateParams as ActionTypeConfigUpdateParams
from .transformation_list_errors_params import TransformationListErrorsParams as TransformationListErrorsParams
from .transformation_list_errors_response import TransformationListErrorsResponse as TransformationListErrorsResponse
from .action_type_config_create_base_param import ActionTypeConfigCreateBaseParam as ActionTypeConfigCreateBaseParam
from .action_type_config_upsert_base_param import ActionTypeConfigUpsertBaseParam as ActionTypeConfigUpsertBaseParam
from .split_config_semantic_page_item_class import SplitConfigSemanticPageItemClass as SplitConfigSemanticPageItemClass
from .transformation_deprecated_update_params import (
    TransformationDeprecatedUpdateParams as TransformationDeprecatedUpdateParams,
)
from .split_config_semantic_page_item_class_param import (
    SplitConfigSemanticPageItemClassParam as SplitConfigSemanticPageItemClassParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SubscriptionCreateParams"]


class SubscriptionCreateParams(TypedDict, total=False):
    action_type_config_id: Required[Annotated[str, PropertyInfo(alias="actionTypeConfigID")]]
    """Unique identifier of action this subscription listens to."""

    name: Required[str]
    """Name of subscription."""

    type: Required[Literal["transform", "route", "split_collection", "split_item", "error", "join"]]
    """Type of subscription."""

    disabled: bool
    """Toggles whether subscription is active or not."""

    webhook_url: Annotated[str, PropertyInfo(alias="webhookURL")]
    """URL bem will send webhook requests to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TransformationListErrorsResponse", "TransformationError"]


class TransformationError(BaseModel):
    message: str
    """
    Message describing the error encountered while attempting to transform the
    original input data.
    """

    pipeline_id: str = FieldInfo(alias="pipelineID")
    """ID of pipeline that was used to attempt to transform the original input data."""

    reference_id: str = FieldInfo(alias="referenceID")
    """The unique ID you use internally to refer to this data point."""

    timestamp: datetime
    """Timestamp indicating when the transformation was created."""

    transformation_error_id: str = FieldInfo(alias="transformationErrorID")
    """Unique ID of this error generated by bem following Segment's KSUID conventions."""


class TransformationListErrorsResponse(BaseModel):
    total_count: Optional[int] = FieldInfo(alias="totalCount", default=None)
    """The total number of results available."""

    transformation_errors: Optional[List[TransformationError]] = FieldInfo(alias="transformationErrors", default=None)
    """An array of objects containing the transformation errors."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["EmailActionBaseParam"]


class EmailActionBaseParam(TypedDict, total=False):
    template_variables: Required[Annotated[object, PropertyInfo(alias="templateVariables")]]
    """Template variables to be used in the email body and subject.

    Templates are injected into the email body and subject as
    `{{template_variable}}`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .input_type import InputType

__all__ = ["SplitActionBase"]


class SplitActionBase(BaseModel):
    input_content: str = FieldInfo(alias="inputContent")
    """The file content you want to split as a base64 encoded string.

    If the `inputType` is `email`, this is equivalent to the raw format returned by
    the Gmail API.
    """

    input_type: InputType = FieldInfo(alias="inputType")
    """The input type of the content you're sending for transformation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated

from .._utils import PropertyInfo
from .action_type_config_upsert_base_param import ActionTypeConfigUpsertBaseParam

__all__ = ["UpsertJoinConfigParam"]


class UpsertJoinConfigParam(ActionTypeConfigUpsertBaseParam, total=False):
    join_type: Annotated[Literal["standard"], PropertyInfo(alias="joinType")]
    """The type of join to perform."""

    next_action_type_config_id: Annotated[str, PropertyInfo(alias="nextActionTypeConfigID")]
    """Unique identifier of action type config to run after join."""

    output_schema: Annotated[object, PropertyInfo(alias="outputSchema")]
    """Desired output structure defined in standard JSON Schema convention."""

    output_schema_name: Annotated[str, PropertyInfo(alias="outputSchemaName")]
    """Name of output schema object."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, List, Iterable, cast
from typing_extensions import Literal, overload

import httpx

from ..types import (
    ActionType,
    action_type_config_list_params,
    action_type_config_create_params,
    action_type_config_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.action_type import ActionType
from ..types.action_type_config import ActionTypeConfig
from ..types.route_list_item_param import RouteListItemParam
from ..types.action_type_config_list_response import ActionTypeConfigListResponse

__all__ = ["ActionTypeConfigsResource", "AsyncActionTypeConfigsResource"]


class ActionTypeConfigsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionTypeConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ActionTypeConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionTypeConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return ActionTypeConfigsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        body: action_type_config_create_params.CreateTransformConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        body: action_type_config_create_params.CreateRouteConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        body: action_type_config_create_params.CreateSplitConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        body: action_type_config_create_params.CreateJoinConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        body: action_type_config_create_params.CreateEmailConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["body"])
    def create(
        self,
        *,
        body: action_type_config_create_params.CreateTransformConfigBody
        | action_type_config_create_params.CreateRouteConfigBody
        | action_type_config_create_params.CreateSplitConfigBody
        | action_type_config_create_params.CreateJoinConfigBody
        | action_type_config_create_params.CreateEmailConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        return cast(
            ActionTypeConfig,
            self._post(
                "/v1-alpha/action-type-configs",
                body=maybe_transform(body, action_type_config_create_params.ActionTypeConfigCreateParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ActionTypeConfig),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def retrieve(
        self,
        action_type_config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Get an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not action_type_config_id:
            raise ValueError(
                f"Expected a non-empty value for `action_type_config_id` but received {action_type_config_id!r}"
            )
        return cast(
            ActionTypeConfig,
            self._get(
                f"/v1-alpha/action-type-configs/{action_type_config_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ActionTypeConfig),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    def update(
        self,
        action_type_config_id: str,
        *,
        complex_tabular_transform_enabled: bool | NotGiven = NOT_GIVEN,
        independent_document_processing_enabled: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        next_action_type_config_id: str | NotGiven = NOT_GIVEN,
        output_schema: object | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Update an Action Type Config

        Args:
          complex_tabular_transform_enabled: Whether complex tabular transforms are enabled on the pipeline. This enables the
              pipeline to parse CSVs with multiple tables in the same file, and to transpose
              CSVs that can't be parsed row-wise.

          independent_document_processing_enabled: Whether independent transformations is enabled. For PDFs sent through the
              pipeline, this enables independent transformations for each individual page. For
              CSVs, this enables transforming chunks of rows in the CSV.

          next_action_type_config_id: Unique identifier of action type config to run after transformation. Currently
              only email is supported.

          output_schema: Desired output structure defined in standard JSON Schema convention.

          output_schema_name: Name of output schema object.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        action_type_config_id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        routes: Iterable[RouteListItemParam] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """Update an Action Type Config

        Args:
          description: Description of router.

        Can be used to provide additional context on router's
              purpose and expected inputs.

          routes: List of routes.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        action_type_config_id: str,
        *,
        name: str | NotGiven = NOT_GIVEN,
        print_page_split_config: action_type_config_update_params.UpsertSplitConfigPrintPageSplitConfig
        | NotGiven = NOT_GIVEN,
        semantic_page_split_config: action_type_config_update_params.UpsertSplitConfigSemanticPageSplitConfig
        | NotGiven = NOT_GIVEN,
        split_type: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Update an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        action_type_config_id: str,
        *,
        join_type: Literal["standard"] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        next_action_type_config_id: str | NotGiven = NOT_GIVEN,
        output_schema: object | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Update an Action Type Config

        Args:
          join_type: The type of join to perform.

          next_action_type_config_id: Unique identifier of action type config to run after join.

          output_schema: Desired output structure defined in standard JSON Schema convention.

          output_schema_name: Name of output schema object.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        action_type_config_id: str,
        *,
        body: str | NotGiven = NOT_GIVEN,
        from_email: str | NotGiven = NOT_GIVEN,
        from_name: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        subject: str | NotGiven = NOT_GIVEN,
        to_email: str | NotGiven = NOT_GIVEN,
        to_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """Update an Action Type Config

        Args:
          body: Body of the email.

        This can be HTML, and include template variables in the form
              of `{{template_variable}}`. Template variables are taken from the output of the
              transformation.

          from_email: Email address to send the email from.

          from_name: Name of the sender.

          subject: Subject of the email. This can include template variables in the form of
              `{{template_variable}}`. Template variables are taken from the output of the
              transformation.

          to_email: Email address to send the email to.

          to_name: Name of the recipient.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    def update(
        self,
        action_type_config_id: str,
        *,
        complex_tabular_transform_enabled: bool | NotGiven = NOT_GIVEN,
        independent_document_processing_enabled: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        next_action_type_config_id: str | NotGiven = NOT_GIVEN,
        output_schema: object | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        routes: Iterable[RouteListItemParam] | NotGiven = NOT_GIVEN,
        print_page_split_config: action_type_config_update_params.UpsertSplitConfigPrintPageSplitConfig
        | NotGiven = NOT_GIVEN,
        semantic_page_split_config: action_type_config_update_params.UpsertSplitConfigSemanticPageSplitConfig
        | NotGiven = NOT_GIVEN,
        split_type: str | NotGiven = NOT_GIVEN,
        join_type: Literal["standard"] | NotGiven = NOT_GIVEN,
        body: str | NotGiven = NOT_GIVEN,
        from_email: str | NotGiven = NOT_GIVEN,
        from_name: str | NotGiven = NOT_GIVEN,
        subject: str | NotGiven = NOT_GIVEN,
        to_email: str | NotGiven = NOT_GIVEN,
        to_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        if not action_type_config_id:
            raise ValueError(
                f"Expected a non-empty value for `action_type_config_id` but received {action_type_config_id!r}"
            )
        return cast(
            ActionTypeConfig,
            self._patch(
                f"/v1-alpha/action-type-configs/{action_type_config_id}",
                body=maybe_transform(
                    {
                        "complex_tabular_transform_enabled": complex_tabular_transform_enabled,
                        "independent_document_processing_enabled": independent_document_processing_enabled,
                        "name": name,
                        "next_action_type_config_id": next_action_type_config_id,
                        "output_schema": output_schema,
                        "output_schema_name": output_schema_name,
                        "description": description,
                        "routes": routes,
                        "print_page_split_config": print_page_split_config,
                        "semantic_page_split_config": semantic_page_split_config,
                        "split_type": split_type,
                        "join_type": join_type,
                        "body": body,
                        "from_email": from_email,
                        "from_name": from_name,
                        "subject": subject,
                        "to_email": to_email,
                        "to_name": to_name,
                    },
                    action_type_config_update_params.ActionTypeConfigUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ActionTypeConfig),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        action_type: ActionType | NotGiven = NOT_GIVEN,
        action_type_config_ids: List[str] | NotGiven = NOT_GIVEN,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfigListResponse:
        """
        List Action Type Configs

        Args:
          action_type: The type of the action.

          ending_before: A cursor to use in pagination. `endingBefore` is an action type config ID that
              defines your place in the list.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is an action type config ID that
              defines your place in the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1-alpha/action-type-configs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "action_type": action_type,
                        "action_type_config_ids": action_type_config_ids,
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    action_type_config_list_params.ActionTypeConfigListParams,
                ),
            ),
            cast_to=ActionTypeConfigListResponse,
        )

    def delete(
        self,
        action_type_config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not action_type_config_id:
            raise ValueError(
                f"Expected a non-empty value for `action_type_config_id` but received {action_type_config_id!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1-alpha/action-type-configs/{action_type_config_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncActionTypeConfigsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionTypeConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionTypeConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionTypeConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/bem-team/bem-sdk-python#with_streaming_response
        """
        return AsyncActionTypeConfigsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        body: action_type_config_create_params.CreateTransformConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        body: action_type_config_create_params.CreateRouteConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        body: action_type_config_create_params.CreateSplitConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        body: action_type_config_create_params.CreateJoinConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        body: action_type_config_create_params.CreateEmailConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Create an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["body"])
    async def create(
        self,
        *,
        body: action_type_config_create_params.CreateTransformConfigBody
        | action_type_config_create_params.CreateRouteConfigBody
        | action_type_config_create_params.CreateSplitConfigBody
        | action_type_config_create_params.CreateJoinConfigBody
        | action_type_config_create_params.CreateEmailConfigBody,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        return cast(
            ActionTypeConfig,
            await self._post(
                "/v1-alpha/action-type-configs",
                body=await async_maybe_transform(body, action_type_config_create_params.ActionTypeConfigCreateParams),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ActionTypeConfig),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def retrieve(
        self,
        action_type_config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Get an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not action_type_config_id:
            raise ValueError(
                f"Expected a non-empty value for `action_type_config_id` but received {action_type_config_id!r}"
            )
        return cast(
            ActionTypeConfig,
            await self._get(
                f"/v1-alpha/action-type-configs/{action_type_config_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ActionTypeConfig),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    async def update(
        self,
        action_type_config_id: str,
        *,
        complex_tabular_transform_enabled: bool | NotGiven = NOT_GIVEN,
        independent_document_processing_enabled: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        next_action_type_config_id: str | NotGiven = NOT_GIVEN,
        output_schema: object | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Update an Action Type Config

        Args:
          complex_tabular_transform_enabled: Whether complex tabular transforms are enabled on the pipeline. This enables the
              pipeline to parse CSVs with multiple tables in the same file, and to transpose
              CSVs that can't be parsed row-wise.

          independent_document_processing_enabled: Whether independent transformations is enabled. For PDFs sent through the
              pipeline, this enables independent transformations for each individual page. For
              CSVs, this enables transforming chunks of rows in the CSV.

          next_action_type_config_id: Unique identifier of action type config to run after transformation. Currently
              only email is supported.

          output_schema: Desired output structure defined in standard JSON Schema convention.

          output_schema_name: Name of output schema object.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        action_type_config_id: str,
        *,
        description: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        routes: Iterable[RouteListItemParam] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """Update an Action Type Config

        Args:
          description: Description of router.

        Can be used to provide additional context on router's
              purpose and expected inputs.

          routes: List of routes.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        action_type_config_id: str,
        *,
        name: str | NotGiven = NOT_GIVEN,
        print_page_split_config: action_type_config_update_params.UpsertSplitConfigPrintPageSplitConfig
        | NotGiven = NOT_GIVEN,
        semantic_page_split_config: action_type_config_update_params.UpsertSplitConfigSemanticPageSplitConfig
        | NotGiven = NOT_GIVEN,
        split_type: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Update an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        action_type_config_id: str,
        *,
        join_type: Literal["standard"] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        next_action_type_config_id: str | NotGiven = NOT_GIVEN,
        output_schema: object | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """
        Update an Action Type Config

        Args:
          join_type: The type of join to perform.

          next_action_type_config_id: Unique identifier of action type config to run after join.

          output_schema: Desired output structure defined in standard JSON Schema convention.

          output_schema_name: Name of output schema object.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        action_type_config_id: str,
        *,
        body: str | NotGiven = NOT_GIVEN,
        from_email: str | NotGiven = NOT_GIVEN,
        from_name: str | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        subject: str | NotGiven = NOT_GIVEN,
        to_email: str | NotGiven = NOT_GIVEN,
        to_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        """Update an Action Type Config

        Args:
          body: Body of the email.

        This can be HTML, and include template variables in the form
              of `{{template_variable}}`. Template variables are taken from the output of the
              transformation.

          from_email: Email address to send the email from.

          from_name: Name of the sender.

          subject: Subject of the email. This can include template variables in the form of
              `{{template_variable}}`. Template variables are taken from the output of the
              transformation.

          to_email: Email address to send the email to.

          to_name: Name of the recipient.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    async def update(
        self,
        action_type_config_id: str,
        *,
        complex_tabular_transform_enabled: bool | NotGiven = NOT_GIVEN,
        independent_document_processing_enabled: bool | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        next_action_type_config_id: str | NotGiven = NOT_GIVEN,
        output_schema: object | NotGiven = NOT_GIVEN,
        output_schema_name: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        routes: Iterable[RouteListItemParam] | NotGiven = NOT_GIVEN,
        print_page_split_config: action_type_config_update_params.UpsertSplitConfigPrintPageSplitConfig
        | NotGiven = NOT_GIVEN,
        semantic_page_split_config: action_type_config_update_params.UpsertSplitConfigSemanticPageSplitConfig
        | NotGiven = NOT_GIVEN,
        split_type: str | NotGiven = NOT_GIVEN,
        join_type: Literal["standard"] | NotGiven = NOT_GIVEN,
        body: str | NotGiven = NOT_GIVEN,
        from_email: str | NotGiven = NOT_GIVEN,
        from_name: str | NotGiven = NOT_GIVEN,
        subject: str | NotGiven = NOT_GIVEN,
        to_email: str | NotGiven = NOT_GIVEN,
        to_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfig:
        if not action_type_config_id:
            raise ValueError(
                f"Expected a non-empty value for `action_type_config_id` but received {action_type_config_id!r}"
            )
        return cast(
            ActionTypeConfig,
            await self._patch(
                f"/v1-alpha/action-type-configs/{action_type_config_id}",
                body=await async_maybe_transform(
                    {
                        "complex_tabular_transform_enabled": complex_tabular_transform_enabled,
                        "independent_document_processing_enabled": independent_document_processing_enabled,
                        "name": name,
                        "next_action_type_config_id": next_action_type_config_id,
                        "output_schema": output_schema,
                        "output_schema_name": output_schema_name,
                        "description": description,
                        "routes": routes,
                        "print_page_split_config": print_page_split_config,
                        "semantic_page_split_config": semantic_page_split_config,
                        "split_type": split_type,
                        "join_type": join_type,
                        "body": body,
                        "from_email": from_email,
                        "from_name": from_name,
                        "subject": subject,
                        "to_email": to_email,
                        "to_name": to_name,
                    },
                    action_type_config_update_params.ActionTypeConfigUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ActionTypeConfig),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def list(
        self,
        *,
        action_type: ActionType | NotGiven = NOT_GIVEN,
        action_type_config_ids: List[str] | NotGiven = NOT_GIVEN,
        ending_before: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        sort_order: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        starting_after: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ActionTypeConfigListResponse:
        """
        List Action Type Configs

        Args:
          action_type: The type of the action.

          ending_before: A cursor to use in pagination. `endingBefore` is an action type config ID that
              defines your place in the list.

          sort_order: Specifies sorting behavior. The two options are `asc` and `desc` to sort
              ascending and descending respectively, with default sort being ascending. Paging
              works in both directions.

          starting_after: A cursor to use in pagination. `startingAfter` is an action type config ID that
              defines your place in the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1-alpha/action-type-configs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "action_type": action_type,
                        "action_type_config_ids": action_type_config_ids,
                        "ending_before": ending_before,
                        "limit": limit,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                    },
                    action_type_config_list_params.ActionTypeConfigListParams,
                ),
            ),
            cast_to=ActionTypeConfigListResponse,
        )

    async def delete(
        self,
        action_type_config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Delete an Action Type Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not action_type_config_id:
            raise ValueError(
                f"Expected a non-empty value for `action_type_config_id` but received {action_type_config_id!r}"
            )
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1-alpha/action-type-configs/{action_type_config_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ActionTypeConfigsResourceWithRawResponse:
    def __init__(self, action_type_configs: ActionTypeConfigsResource) -> None:
        self._action_type_configs = action_type_configs

        self.create = to_raw_response_wrapper(
            action_type_configs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            action_type_configs.retrieve,
        )
        self.update = to_raw_response_wrapper(
            action_type_configs.update,
        )
        self.list = to_raw_response_wrapper(
            action_type_configs.list,
        )
        self.delete = to_raw_response_wrapper(
            action_type_configs.delete,
        )


class AsyncActionTypeConfigsResourceWithRawResponse:
    def __init__(self, action_type_configs: AsyncActionTypeConfigsResource) -> None:
        self._action_type_configs = action_type_configs

        self.create = async_to_raw_response_wrapper(
            action_type_configs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            action_type_configs.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            action_type_configs.update,
        )
        self.list = async_to_raw_response_wrapper(
            action_type_configs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            action_type_configs.delete,
        )


class ActionTypeConfigsResourceWithStreamingResponse:
    def __init__(self, action_type_configs: ActionTypeConfigsResource) -> None:
        self._action_type_configs = action_type_configs

        self.create = to_streamed_response_wrapper(
            action_type_configs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            action_type_configs.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            action_type_configs.update,
        )
        self.list = to_streamed_response_wrapper(
            action_type_configs.list,
        )
        self.delete = to_streamed_response_wrapper(
            action_type_configs.delete,
        )


class AsyncActionTypeConfigsResourceWithStreamingResponse:
    def __init__(self, action_type_configs: AsyncActionTypeConfigsResource) -> None:
        self._action_type_configs = action_type_configs

        self.create = async_to_streamed_response_wrapper(
            action_type_configs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            action_type_configs.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            action_type_configs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            action_type_configs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            action_type_configs.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["RouteListItemParam", "Origin", "OriginEmail", "Regex"]


class OriginEmail(TypedDict, total=False):
    patterns: List[str]
    """
    Regular expression to match against the email address, using the Go (RE2) regex
    syntax.
    """


class Origin(TypedDict, total=False):
    email: OriginEmail


class Regex(TypedDict, total=False):
    patterns: List[str]
    """Regular expression to match against the input, using the Go (RE2) regex syntax."""


class RouteListItemParam(TypedDict, total=False):
    name: Required[str]
    """Name of route.

    Should indicate what the input type or intent is to appropriately analyze input.
    """

    action_type_config_id: Annotated[str, PropertyInfo(alias="actionTypeConfigID")]
    """ID of action type config to run after routing.

    Currently only Transform Configs are supported.
    """

    description: str
    """Description of route.

    Can be used to provide additional context on route's purpose.
    """

    origin: Origin
    """The origin of the route."""

    regex: Regex
    """Regex to match against the input."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TransformationListParams"]


class TransformationListParams(TypedDict, total=False):
    ending_before: Annotated[str, PropertyInfo(alias="endingBefore")]
    """A cursor to use in pagination.

    `endingBefore` is a transform ID that defines your place in the list. For
    example, if you make a list request and receive 50 objects, starting with
    `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call can include
    `endingBefore=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the previous page of the
    list.
    """

    item_offset: Annotated[int, PropertyInfo(alias="itemOffset")]
    """Filters based on the index of the page of the document you had transformed.

    For CSV transformations, this offset represents starting row of the CSV for
    which you want to fetch transformations for. For PDF transformations, this
    offset represents the starting page of the PDF. This applies for a specific
    reference ID of a transformation. Note that this only applies if you specify one
    reference ID in `referenceIDs` and if you have
    `independentDocumentProcessingEnabled` set to `true` in your pipeline
    configuration. Note that this is zero-indexed.
    """

    limit: int
    """
    This specifies a limit on the number of objects to return, ranging between 1
    and 100.
    """

    pipeline_id: Annotated[str, PropertyInfo(alias="pipelineID")]
    """The unique ID for a given pipeline.

    Will filter to just the transformations processed by the given pipeline. If left
    out, will query over ALL transformations for your account.
    """

    published: bool
    """
    Boolean that toggles filtering whether or not transformations were successfully
    published via webhook.
    """

    published_after: Annotated[Union[str, datetime], PropertyInfo(alias="publishedAfter", format="iso8601")]
    """
    Filters to transformations successfully published via webhook after the
    specified date. Must be in RFC 3339 format.
    """

    published_before: Annotated[Union[str, datetime], PropertyInfo(alias="publishedBefore", format="iso8601")]
    """
    Filters to transformations successfully published via webhook before the
    specified date. Must be in RFC 3339 format.
    """

    reference_ids: Annotated[List[str], PropertyInfo(alias="referenceIDs")]
    """Array with the reference IDs of the transformed data points.

    Formatted as a CSV array.
    """

    sort_order: Annotated[Literal["asc", "desc"], PropertyInfo(alias="sortOrder")]
    """Specifies sorting behavior.

    The two options are `asc` and `desc` to sort ascending and descending
    respectively, with default sort being ascending. Paging works in both
    directions.
    """

    starting_after: Annotated[str, PropertyInfo(alias="startingAfter")]
    """A cursor to use in pagination.

    `startingAfter` is a transform ID that defines your place in the list. For
    example, if you make a list request and receive 50 objects, ending with
    `tr_2bxoJPNdSD4LgRT4YVC4gt72hlI`, your subsequent call can include
    `startingAfter=tr_2bxoJPNdSD4LgRT4YVC4gt72hlI` to fetch the next page of the
    list.
    """

    transformation_ids: Annotated[List[str], PropertyInfo(alias="transformationIDs")]
    """Array with the transform IDs of the transformed data points.

    Formatted as a CSV array.
    """
